"""Pyamq Sender Class

This module provides a Consumer Class which sends Tests via PyAMQ.

Author: Thomas Low <thomas.low@cern.ch>
"""

import time, re
try:
	import logging
except:
	import swatc.logging as logging
log = logging.getLogger(__name__)

from swatc.common.testconsumer import TestConsumer
from swatc.common.test import Test
from swatc.common import openssl, test
from swatc.common.pyamq.producer import Producer


class PyamqSender(TestConsumer):
	"""This module converts test results to a activeMQ message and sends it."""
	
	def __init__(self, 	broker, 
						destination, 
						connections	= 1, 
						random		= 0, 
						timeout		= None, 
						pubkey		= None,
						testfilter	= None):
		"""Inits the Sender with a set of arguments.
		
		BROKER space seperated string of connection-strings
		DESTINATION destination where to send the messages
		CONNECTIONS number of multiple connections
		RANDOM if true, list of brokers will be processed randomly
		TIMEOUT maximum timeout the sender will run after start
		PUBKEY public key which will be used to encrypt messages
		TESTFILTER a regular expression for the test name
		"""
		self.broker = broker.split(" ")
		self.destination = destination
		self.connections = int(connections)
		self.random = int(random)
		self.timeout = float(timeout)
		self.endtime = time.time()+self.timeout
		self.pubkey = pubkey
		if testfilter:
			self.testpattern = re.compile(testfilter)
		else:
			self.testpattern = None
		self.pyamqid = test._getUniqueId()
		
	def setup(self):
		self.producer = Producer(self.broker, self.connections, 
											self.random)
		
	def consume(self, test):
		"""Sends a test."""
		#ignore tests which do not match the filter
		if self.testpattern and not self.testpattern.match(test.testName):
			log.debug("PyamqSender: \"%s\" skipped" % test.testName)
			return
		
		test.pyamqId = self.pyamqid
		msg = str(test)
		
		#encrypt message with pyamqEncrypt argument
		if test.has_key("pyamqEncrypt") and test.pyamqEncrypt == '1':
			msg = openssl.encrypt(msg, self.pubkey)
			log.debug("PyamqSender: \"%s\" encrypted" % test.testName)
			
		dest = self.destination
		#redirect messages with pyamqSubtopic argument
		if test.has_key("pyamqSubtopic"):
			dest = dest + "." + test.pyamqSubtopic
		
		#send message
		self.producer.sendMessage({
			'header'		: {'destination': dest},
			'body'		 	: msg
		})
		
	def cleanup(self):
		self.producer.wait(self.endtime-time.time())
		p,s = self.producer.stop()
		if p: #still pending messages
			log.warning("PyamqSender: %d out of %d Message not sent" % (len(p),len(p+s)))
