"""Process Producer Class

This module provides a Process-Producer Class to create new tests.

ProcTester executes a set of executables within directories and creates tests
from their stdout.

Author: Thomas Low <thomas.low@cern.ch>
Edited by: Danica Stojiljkovic <danica@scl.rs>
"""

import sys, os, time, random, os.path, tempfile
try:
	import logging
except:
	import swatc.logging as logging
log = logging.getLogger(__name__)

from swatc.common.testproducer import TestProducer
from swatc.common import test, openssl, tar, message, processmgr
from swatc.common.test import Test
from swatc.clients.wn import util
import memory

class ProcTester(TestProducer):
	"""This Producer will call a set of executables in directory.
	
	Therefore it lookes for a filename for which an executable file and a file
	filename.testdef exists. The tuples within that file will be loaded into a
	test. Then the file will be executed and the stdout will be merged into the
	test together with some basic information like starttime, endtime, exitcode.
	The stderr will be redirected to the detailsdata-field of the test.
	"""
	def __init__(self, 
					directories,
					timeout = None,
					threads = 3,
					memoryFile = "/tmp/glite-swat-client-wn.mem",
					forceexecution = 0 ):
		"""Just saves the arguments and does some initiating.
		
		DIRECTORIES is a list of paths to directories separated by a colon
		TIMEOUT is the time in seconds after all processes will be killed
		THREADS is the number of simultaneous subprocesses
		MEMORYFILE is the path to the memory file, so tests can be scheduled
		FORCEEXECUTION is flag which forces all tests to be produced if set to 1
		"""
		self.directories = directories.split(":")
		self.timeout = float(timeout)
		self.endtime = time.time() + self.timeout
		self.threads = int(threads)
		self.tests = {}
		self.event = None
		self.memoryFile = memoryFile
		self.memory = None
		self.forceExecution = forceexecution
		
	def setup(self):
		"""Loads the test from the directories.
		
		Merges the *.testdef files with a default set of tuples (testdef.default) 
		and stores them in self.tests.
		"""
		mem = memory.TestMemory(self.memoryFile)
		for directory in self.directories:
			if os.path.isdir(directory):
				#get default test of directory
				dTest_path = os.path.join(directory, "testdef.default")
				dTest = self._createDefaultTestFromFile(dTest_path)
				for f in os.listdir(directory):
					path = os.path.join(directory,f)
					if f[-8:] == '.testdef' and os.access(path, os.F_OK) \
							and not os.access(path, os.X_OK):
						#for each *.testdef create a test
						try:
							t = self._createTestFromFile(path, dTest, mem)
							if t:
								self.tests[t.executionCommand] = t
						except Exception, err:
							log.error("%s : %s" % (os.path.basename(path), err))
			else:
				log.error("directory %s does not exist" % directory)
	
	def _createDefaultTestFromFile(self, path):
		if not os.path.exists(path):
			return message.Message()
		try:
			lines = open(path, 'r').readlines()
			return message.stringToMessage("".join(lines))
		except Exception, err:
			log.error("%s : %s" % (os.path.basename(path), err))
			return message.Message()
	
	def _createTestFromFile(self, path, dTest, mem):
		lines = open(path, 'r').readlines()
		tmp = test.stringToTest("".join(lines))
		t = Test(tmp.testName)
		t.update(message.messageToDict(dTest))
		t.update(test.testToDict(tmp))

		#setup the command which will be executed
		if t.has_key('executionFile'):
			pathExec = os.path.dirname(path)+'/'+t.executionFile
		else:
			#try to guess executionFile
			pathExec = os.path.dirname(path)+'/'+os.path.basename(path).replace('.testdef','')
		if not os.access(pathExec, os.X_OK):
			raise ValueError("executionFile is not executable: %s" % pathExec)
		t.executionCommand = pathExec
		if t.has_key('executionArguments'):
			t.executionCommand += " "+t.executionArguments	
		
		#checks if option for forcing execution is on
		if self.forceExecution == "1":
			return t

		if t.has_key('forceExecution'):
			if t.forceExecution == '1':
				return t

		#Schedule if interval is set and memory is working
		scheduled = 0
		if t.has_key('executionInterval'):
			offset = int(t.executionInterval)
			if mem.has_key(t.testName):
				scheduled = 1
				if mem[t.testName]+offset > int(time.time()):
					log.debug("%s next schedule in %d seconds" % \
					(os.path.basename(t.executionCommand), int(time.time())-mem[t.testName]-offset))
					return None 
		
		#compute execution probability if not scheduled by memory
		if not scheduled and t.has_key('executionProbability'):
			prob = float(t.executionProbability)
			if prob < 0 or prob > 1:
				raise ValueError("value not between 0 and 1")
			rand = random.random()
			if prob < rand:
				log.debug("%s lost game (%.3f to %.3f)" % \
					(os.path.basename(t.executionCommand), prob, rand))
				return None
		return t 
		
		
	def _convertProcToTest(self, cmd, data):
		"""Gets results of a subprocess and converts them into a test-class."""
		#find test-instace with the matching command
		t = self.tests.get(cmd)
		if not t:
			log.error("process could not be assigned to a test")
			return
	
		#setup some basic attributes
		t.hostname	= util.hostname
		t.hostid	= util.hostid
		t.executionUnit = "ProcTester"
		t.executionExitcode	= str(data['exitcode'])
		t.executionExitsignal= str(data['signal'])
		t.executionStarttime = repr(data['starttime'])
		t.executionTime = repr(data['endtime']-data['starttime'])
		t.executionEndtime 	= repr(data['endtime'])
		
		#convert stdout of test into arguments for the test-class
		errors = []
		lines = data['stdout'].split('\n')
		for line in lines:
			if line == '':
				continue
			kv = line.split(':',1)
			if len(kv) == 2:
				try:
					setattr(t,kv[0].strip(),kv[1].strip())
				except ValueError, err:
					log.error("[%s] ValueError: %s (key: %s, value: %s)" % \
						(t.testName, err, repr(kv[0]), repr(kv[1])))
					errors.append("ValueError: %s (key: %s, value: %s)" % \
						(t.testName, err, repr(kv[0]), repr(kv[1])))
	
		if data['stderr']:
			t.detailsdata = str(data['stderr'])
		if errors:
			if not t.has_key('detailsdata'):
				t.detailsdata = ''
			t.detailsdata += 'Format Errors:\n'+'\n'.join(errors)	
		self.event(t)
	
	
	def run(self, event):
		"""Runs all loaded tests with their specified timeout."""
		self.event = event
		if self.tests:
			self.testMgr = processmgr.ProcessManager(self.threads)
			for t in self.tests.values():
				try:
					timeout = min(float(t.executionTimeout),self.endtime-time.time())
				except:
					timeout = self.endtime-time.time()
				self.testMgr.execute(t.executionCommand, self._convertProcToTest, timeout)
			
	
	def cleanup(self):
		"""Waits until all tests are done or time is over."""
		if self.tests:
			self.testMgr.wait(self.endtime-time.time())
			self.testMgr.stop()

