"""Signed Tar Producer

This module uses ProcTester and PyTester to run tests from a signed tar ball.

It scans directories and verifyies signed *.tar.smime files. If valid, it 
extracts the content into a temporary directory and starts PyTester and 
ProcTester using this directory.

Author: Thomas Low <thomas.low@cern.ch>
"""

import sys, os, time, random, os.path, tempfile
try:
	import logging
except:
	import swatc.logging as logging
log = logging.getLogger(__name__)

from swatc.common.testproducer import TestProducer
from swatc.common import test, openssl, tar

from swatc.clients.wn import util

from proctester import ProcTester
from pytester import PyTester

class SignedTarTester(TestProducer):
	
	def __init__(self, 
					directories,
					certs,
					allowed,
					timeout = None,
					threads = 3,
					forceexecution = 0):
		"""Stores arguments and inits some variables.
		
		DIRECTORIES is a list of directories separated by colons.
		CERTS is the directory which contains the certificate of the CA.
		ALLOWED is a list of DNs separated by colon which are allowed to sign tests
		TIMEOUT is the timeout for the ProcTester
		THREADS is the number of subprocesses for the ProcTester
		"""
		self.directories = directories.split(':')
		self.certs = certs
		self.allowed = allowed.split(':')
		self.timeout = timeout
		self.threads = threads
		
		self.tmpDirs = []
		self.pathlist = []
		self.data = []
		self.dirlist = []
		self.dir_signer_map = {}
		self.pyTester = None
		self.procTester = None
		self.event = None
		self.starttime = time.time()
		self.forceExecution = forceexecution
		
	def setup(self):
		"""Looks for signed tar balls."""
		#create list of tar balls
		for directory in self.directories:
			if os.path.isdir(directory):
				for f in os.listdir(directory):
					path = os.path.join(directory,f)
					if f[-10:] == '.tar.smime' and os.access(path, os.F_OK) \
							and not os.access(path, os.X_OK):
						log.debug("Try to verify %s" % str(f))
						self.pathlist.append(path)
			else:
				log.info("directory %s does not exist." % str(directory))
				self.data.append("ERROR: directory %s does not exist." % str(directory))
	
	def run(self, event):
		self.event = event
		#go through each tar ball
		for path in self.pathlist:
			#try to verify it
			tarFile = openssl._mkstemp()
			tarDir  = openssl._mkdtemp()
			self.tmpDirs.append(tarDir)
			try:
				cert = openssl.verifyFile(path, tarFile, self.certs)
				#find out subject and check against allowed
				subj = openssl.x509subject(cert)
				if subj in self.allowed:
					#extract tar ball to tempdirectory
					tar.extract(tarFile, tarDir)
					self.dirlist.append(tarDir)
					self.dir_signer_map[tarDir] = subj
				else:
					log.info("signer %s of file %s not in allowed list" % (subj, os.path.basename(path)))
					self.data.append("ERROR: signer %s of file %s not in allowed list" % (subj, os.path.basename(path)))
			except Exception, err:
				log.error("%s : %s" % (os.path.basename(path), err))
			try: os.remove(tarFile)
			except: pass
		
		if self.dirlist: #start PyTester and ProcTester with temp directories
			self.pyTester = PyTester(":".join(self.dirlist))
			self.pyTester.setup()
			self.pyTester.run(self.myEvent)
			
			self.procTester = ProcTester(":".join(self.dirlist), self.timeout, self.threads, forceexecution = self.forceExecution)
			self.procTester.setup()
			self.procTester.run(self.myEvent)
		
	def myEvent(self, t):
		#merge some tuples to provide debug info for signed tests
		if t.has_key("executionCommand"):
			d = os.path.dirname(t.executionCommand)
			t.signerDN = self.dir_signer_map.get(d, "not found")
			if t.has_key("executionUnit"):
				t.executionUnit = "SignedTester (%s)" % t.executionUnit
			else:
				t.executionUnit = "SignedTester (Unknown)"
		self.event(t)
							
	
	def cleanup(self):
		
		if self.pyTester:
			self.pyTester.cleanup()
		
		if self.procTester:
			self.procTester.cleanup()
		
		#delete all temp directories
		for t in self.tmpDirs:
			try:
				os.system('rm -rf  %s ' % t)
			except Exception, err:
				log.warning(err)
